#ifndef CUSTOMGRAPH_H
#define CUSTOMGRAPH_H

#include <QQuickItem>
#include <QQuickWindow>
#include <QSGRendererInterface>
#include <QQuickItem>
#include <QSGRenderNode>
#include <QOpenGLShaderProgram>
#include <QOpenGLBuffer>
#include <QOpenGLFunctions>
#include "../Renderer.h"

class OpenGLRenderNode : public QSGRenderNode
{
	public:

		OpenGLRenderNode(QQuickItem* item) : m_item(item){

		}

		void render(const RenderState *) override
		{
			QOpenGLFunctions *f = QOpenGLContext::currentContext()->functions();
			if(!renderer){
				renderer = new CustomRenderer(f);
				renderer->initialize();

				qDebug() <<  QOpenGLContext::currentContext()->format();
			}
			renderer->render();
		}

		StateFlags changedStates() const override
		{
			return ColorState;
		}

		RenderingFlags flags() const override
		{
			return DepthAwareRendering ;
		}

		QRectF rect() const override
		{
			return QRect(0, 0, m_width, m_height);
		}

		void sync(QQuickItem *item)
		{
			m_width = item->width();
			m_height = item->height();
		}

	private:
		int m_width = 0;
		int m_height = 0;
		QQuickItem * m_item = nullptr;
		CustomRenderer * renderer = nullptr;
};

class CustomGraph : public QQuickItem
{
		Q_OBJECT

	public:
		CustomGraph(QQuickItem *parent = nullptr)
			: QQuickItem(parent)
		{
			// Our item shows something so set the flag.
			setFlag(ItemHasContents);
		}

		QSGNode *updatePaintNode(QSGNode *node, UpdatePaintNodeData *) override
		{
			QSGRenderNode *n = static_cast<QSGRenderNode *>(node);

			QSGRendererInterface *ri = window()->rendererInterface();
			if (!ri)
				return nullptr;

			switch (ri->graphicsApi()) {
				case QSGRendererInterface::OpenGL:
				case QSGRendererInterface::OpenGLRhi:
					if (!n)
						n = new OpenGLRenderNode(this);
					static_cast<OpenGLRenderNode *>(n)->sync(this);
				break;
				default:
				break;
			}

			if (!n)
				qWarning("QSGRendererInterface reports unknown graphics API %d", ri->graphicsApi());

			n->markDirty(QSGNode::DirtyForceUpdate);

			QTimer::singleShot(0, this, &QQuickItem::update);

			return n;
		}
};

#endif // CUSTOMGRAPH_H
