#ifndef CUSTOMGRAPHRENDERER_H
#define CUSTOMGRAPHRENDERER_H

#include <QtGui/qvector3d.h>
#include <QtGui/qmatrix4x4.h>
#include <QtGui/qopenglshaderprogram.h>
#include <QtGui/qopenglfunctions.h>
#include <QElapsedTimer>
#include <QTime>
#include <QVector>
#include <QPainter>
#include <QPaintEngine>
#include <qmath.h>
#include <QElapsedTimer>
#include <QTimer>

#define N_LEN 2048

class CustomRenderer
{
	public:
		CustomRenderer()
		{
			f = new QOpenGLFunctions();
		}

		CustomRenderer(QOpenGLFunctions* f) : f(f)
		{

		}

		void render()
		{
			createGeometry();

			f->glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
			f->glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

			f->glEnable(GL_BLEND);
			f->glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

			QMatrix4x4 modelview;
			modelview.rotate(m_fAngle, 0.0f, 1.0f, 0.0f);
//			modelview.rotate(m_fAngle, 1.0f, 0.0f, 0.0f);
//			modelview.rotate(m_fAngle, 0.0f, 0.0f, 1.0f);
//			modelview.translate(0.0f, -0.2f, 0.0f);

			program1.bind();

			program1.enableAttributeArray(attribute_coord2d);
			program1.setAttributeArray(attribute_coord2d, vertices.constData());
			program1.setUniformValue(uniform_scale_x, scale_x);
			program1.setUniformValue(uniform_offset_x, offset_x);
			program1.setUniformValue(program1.uniformLocation("mv"), modelview);

			f->glDrawArrays(GL_LINE_STRIP, 0, vertices.size());
			program1.disableAttributeArray(attribute_coord2d);

			program1.release();
		}

		void initialize()
		{
			f->initializeOpenGLFunctions();

			const char* vsrc1 =
					"uniform mat4 mv;\n"
					"attribute vec3 coord;\n"
					"varying vec4 f_color;\n"
					"uniform float offset_x;\n"
					"uniform float scale_x;\n"
					"void main(void) {\n"
					"	gl_Position = mv * vec4((coord.x + offset_x) * scale_x, coord.y, coord.z/10., 1);\n"
					"	f_color = vec4(coord.xy / 4.0 + 0.5, scale_x * 1. - mv[0][0], 1);\n"
					"}\n";

			const char* fsrc1 =
					"varying vec4 f_color;\n"
					"void main(void) {\n"
					"	gl_FragColor = f_color;\n"
					"}\n";

			program1.addCacheableShaderFromSourceCode(QOpenGLShader::Vertex, vsrc1);
			program1.addCacheableShaderFromSourceCode(QOpenGLShader::Fragment, fsrc1);
			program1.link();

			attribute_coord2d = program1.attributeLocation("coord");
			uniform_offset_x = program1.uniformLocation("offset_x");
			uniform_scale_x = program1.uniformLocation("scale_x");
		}

		qreal   m_fAngle;

		void createGeometry()
		{
			long el = elapsedTimer.elapsed();
			long diff = el - lastDisplayTime;
			lastDisplayTime = el;

			if (fpsSkip == 60)
			{
				qDebug() << "FPS: " << 1000.0 / (double)(diff);
				fpsSkip = 0;
			}
			else
			{
				fpsSkip++;
			}

			vertices.clear();
			vertices.resize(N_LEN);

			for (int i = 0; i < N_LEN; i++)
			{
				float x = (i - (N_LEN / 2.0)) / 100.0;

				vertices[i].setX(x);
				vertices[i].setY((0.05 * sin(0.5 * x) / x) +  0.001 * (1. - (x * x / 2.0)) +  0.03 * ((rand() % 100) / (100.0)));
				vertices[i].setZ(x);
			}
		}


		QVector<QVector3D> vertices;
		QOpenGLShaderProgram program1;

		GLint attribute_coord2d;
		GLint uniform_offset_x;
		GLint uniform_scale_x;

		float offset_x = 0.0;
		float scale_x = 0.1;
		long lastDisplayTime = 0;
		int fpsSkip = 0 ;
		QElapsedTimer elapsedTimer;
		QOpenGLFunctions* f = nullptr;

};
#endif // CUSTOMGRAPHRENDERER_H
